{{- define "disaster-recovery.script" }}
import os
import sys
import time
import yaml
from datetime import datetime
from kubernetes import client, config
from kubernetes.client.rest import ApiException
from os.path import expanduser
import base64

# Delete for release
import urllib3
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

home_dir = expanduser("~")
kubeconfig_path = os.environ.get("KUBECONFIG", home_dir + "/.kube/config")
destination_namespace = "velero"
source_namespace = os.environ.get("EDP_NAME", "control-plane")

backup = {
    "group": "velero.io",
    "version": "v1",
    "namespace": "velero",
    "plural": "backups"
}


def fetch_latest_backup_name(api_instance, backup_cr, namespace):
    response = api_instance.list_namespaced_custom_object(backup_cr["group"],
                                                          backup_cr["version"],
                                                          backup_cr["namespace"],
                                                          backup_cr["plural"])

    matching_backups = [item for item in response.get('items', []) if
                        item.get("spec", {}).get("includedNamespaces")[0] == namespace and
                        item.get("status", {}).get("phase") == "Completed"]
    latest_backup = max(matching_backups,
                        key=lambda x: datetime.strptime(x.get("metadata", {}).get("creationTimestamp"),
                                                        "%Y-%m-%dT%H:%M:%SZ"))

    return latest_backup.get("metadata", {}).get("name")


def waiting_for_running_pods(api_instance, namespace, status='Ready', timeout_seconds=300):
    start_time = time.time()

    while True:
        try:
            pods = api_instance.list_namespaced_pod(namespace)

            if status == 'Ready':
                all_ready = all(
                    pod.status.phase == "Running" and pod.status.container_statuses[0].ready for pod in pods.items)
            else:
                all_ready = all(pod.status.phase == "Running" for pod in pods.items)

            print(f"Waiting for all pods in namespace {namespace} to be {status}...")

            if all_ready:
                print(f"All pods in namespace {namespace} are {status}.")
                break

            elapsed_time = time.time() - start_time
            if elapsed_time > timeout_seconds:
                print(f"Timeout reached ({timeout_seconds} seconds). Exiting.")
                break

            time.sleep(10)

        except Exception as e:
            print(f"Error occurred: {e}")
            time.sleep(10)


def delete_namespace(client, namespace):
    try:
        client.CoreV1Api().read_namespace(name=namespace)
    except ApiException as e:
        if e.status == 404:
            print(f"Namespace '{namespace}' already deleted. Skip deletion")
            return

    api_instance = client.AppsV1Api()
    deployments = api_instance.list_namespaced_deployment(namespace=namespace)
    print("Scale deployments in " + namespace)
    for deployment in deployments.items:
        deployment.spec.replicas = 0
        api_instance.patch_namespaced_deployment(namespace=namespace, name=deployment.metadata.name, body=deployment)
    api_instance = client.CustomObjectsApi()

    print("Patch CR finalizers for all resources in " + namespace)
    crd_api = client.ApiextensionsV1Api()
    crds = crd_api.list_custom_resource_definition()
    edp_crds = [crd for crd in crds.items if "edp" in crd.metadata.name.lower()]
    for crd in edp_crds:
        resources = api_instance.list_namespaced_custom_object(
            group=crd.spec.group,
            version=crd.spec.versions[0].name,
            namespace=namespace,
            plural=crd.spec.names.plural
        )
        if resources["items"]:
            for resource in resources["items"]:
                print(f"[DEBUG]Removing finalizers from {crd.metadata.name} - {resource['metadata']['name']}")
                resource["metadata"]["finalizers"] = None
                api_instance.patch_namespaced_custom_object(
                    group=crd.spec.group,
                    version=crd.spec.versions[0].name,
                    namespace=namespace,
                    plural=crd.spec.names.plural,
                    name=resource["metadata"]["name"],
                    body=resource,
                )
    api_instance = client.CoreV1Api()
    api_instance.delete_namespace(name=namespace)

    while True:
        try:
            api_instance.read_namespace(name=namespace)
            print(f"Namespace '{namespace}' is still deleting. Waiting...")
            time.sleep(10)
        except ApiException as e:
            if e.status == 404:
                print(f"Namespace '{namespace}' has been deleted successfully.")
                break
            else:
                print(f"Error occurred: {e}")
                break


def restore_namespace(client, namespace, backup_name):
    velero_backup = {"plural": "backups", "version": "v1", "group": "velero.io"}
    velero_restore = {"plural": "restores", "version": "v1", "group": "velero.io"}
    velero_excluded_resources = [
        "nodes",
        "events",
        "events.events.k8s.io",
        "backups.velero.io",
        "restores.velero.io",
        "resticrepositories.velero.io"
    ]
    restore_name = f"{backup_name}-{time.time()}"
    api_instance = client.CustomObjectsApi()
    body = {
        "apiVersion": f"{velero_restore['group']}/{velero_restore['version']}",
        "kind": "Restore",
        "metadata": {
            "name": f"{restore_name}"
        },
        "spec": {
            "backupName": f"{backup_name}",
            "excludedResources": velero_excluded_resources,
            "hooks": {},
            "includedNamespaces": [
                "*"
            ],
        },
    }
    try:
        api_instance.create_namespaced_custom_object(
            body=body,
            namespace="velero",
            plural=velero_restore['plural'],
            version=velero_restore['version'],
            group=velero_restore['group']
        )
    except ApiException as e:
        print(f"error {e}")
    print(f"Restore created {restore_name}")
    time.sleep(5)
    restore = api_instance.get_namespaced_custom_object(
        namespace="velero",
        plural=velero_restore['plural'],
        version=velero_restore['version'],
        group=velero_restore['group'],
        name=restore_name
    )
    print(f"Waiting for restore {restore_name} to be Completed...")
    while restore['status']['phase'] != "Completed":
        restore = api_instance.get_namespaced_custom_object(
            namespace="velero",
            plural=velero_restore['plural'],
            version=velero_restore['version'],
            group=velero_restore['group'],
            name=restore_name
        )
        time.sleep(5)

    print(f"Restore for namespace {namespace} completed")
    api_instance = client.CoreV1Api()
    waiting_for_running_pods(api_instance, namespace, 'Running')

    for pod in api_instance.list_namespaced_pod(namespace=namespace).items:
        if pod.status.phase == "Running":
            print(f"Deleting pod: {pod.metadata.name} in namespace {namespace}")
            api_instance.delete_namespaced_pod(name=pod.metadata.name, namespace=namespace)

    waiting_for_running_pods(api_instance, namespace)


def restore_control_plane_namespace(client, namespace, backup_name):
    v1 = client.CoreV1Api()
    v1Job = client.BatchV1Api()
    delete_options = client.V1DeleteOptions()
    job_name = f"control-plane-restore-{backup_name}"
    job_namespace = "velero"
    job_cm = "control-plane-restore-job-config"

    cm = v1.read_namespaced_config_map(namespace=job_namespace, name=job_cm)
    job_template = cm.data.get("restore-job.yaml")
    job = yaml.safe_load(job_template)

    job["metadata"]["name"] = job_name
    job["spec"]["template"]["spec"].get("containers", {})[0].get("env", {})[0]["value"] = backup_name
    try:
        v1Job.delete_namespaced_job(namespace=job_namespace, name=job_name, body=delete_options)
    except ApiException as e:
        if e.status == 404:
            print(f"Job {job_name} exists. Skipping deletion")
        else:
            sys.exit()

    v1Job.create_namespaced_job(body=job, namespace=job_namespace)

    while True:
        try:
            job = v1Job.read_namespaced_job(name=job_name, namespace=job_namespace)
            if job.status.succeeded is not None and job.status.succeeded > 0:
                print(f"Job '{job_name}' completed successfully in namespace '{job_namespace}'.")

                waiting_for_running_pods(v1, namespace, 'Running')

                for pod in v1.list_namespaced_pod(namespace=namespace).items:
                    if pod.status.phase == "Running":
                        print(f"Deleting pod: {pod.metadata.name} in namespace {namespace}")
                        v1.delete_namespaced_pod(name=pod.metadata.name, namespace=namespace)

                waiting_for_running_pods(v1, namespace)

                return True
        except ApiException as e:
            print(f"Error retrieving Job status: {e}")
            return False


def replace_secrets(client):
    secrets_map = {
        "web-console": {"source_kind": "secret", "source_name": "keycloak-client-web-console-keycloak-client-secret",
                        "target_kind": "secret", "target_name": "keycloak-client-web-console-keycloak-client-secret",
                        "source_jsonpath": "clientSecret", "target_jsonpath": "clientSecret",
                        "source_namespace": "user-management", "target_namespace": "openshift-config",
                        "restart": "false"},
        "group-sync": {"source_kind": "secret", "source_name": "keycloak", "target_kind": "secret",
                       "target_name": "keycloak",
                       "source_jsonpath": "password", "target_jsonpath": "password",
                       "source_namespace": "user-management", "target_namespace": "group-sync-operator",
                       "restart": "true"},
        "control-plane-console": {"source_kind": "oauthclient", "source_name": "admin-console-client",
                                  "target_kind": "secret", "target_name": "oauth-admin-console-client",
                                  "source_jsonpath": "secret", "target_jsonpath": "secret",
                                  "source_namespace": "openshift-config",
                                  "target_namespace": "control-plane", "restart": "true"}}

    for key in secrets_map:
        source_kind = secrets_map.get(key, {})["source_kind"]
        source_namespace = secrets_map.get(key, {})["source_namespace"]
        source_jsonpath = secrets_map.get(key, {})["source_jsonpath"]
        source_name = secrets_map.get(key, {})["source_name"]
        target_kind = secrets_map.get(key, {})["target_kind"]
        target_namespace = secrets_map.get(key, {})["target_namespace"]
        target_jsonpath = secrets_map.get(key, {})["target_jsonpath"]
        target_name = secrets_map.get(key, {})["target_name"]
        restart = secrets_map.get(key, {})["restart"]
        v1 = client.CoreV1Api()

        if source_kind == "secret":
            source_secret = v1.read_namespaced_secret(namespace=source_namespace,
                                                      name=source_name)
            target_secret = v1.read_namespaced_secret(namespace=target_namespace, name=target_name)
            target_secret.data[target_jsonpath] = source_secret.data[source_jsonpath]
            v1.patch_namespaced_secret(namespace=target_namespace, name=target_name, body=target_secret)

            # List all pods in the specified namespace
        elif source_kind == "oauthclient":
            openshift_api = client.CustomObjectsApi()
            oauth_client = openshift_api.get_cluster_custom_object(
                plural="oauthclients",
                version="v1",
                group="oauth.openshift.io",
                name=source_name
            )
            secret_body = v1.read_namespaced_secret(namespace=target_namespace, name=target_name)
            encoded_bytes = base64.b64encode(oauth_client[source_jsonpath].encode("utf-8"))
            encoded_secret = encoded_bytes.decode("utf-8")
            secret_body.data[target_jsonpath] = encoded_secret
            v1.patch_namespaced_secret(namespace=target_namespace, name=target_name, body=secret_body)
        if restart == "true":
            pods = v1.list_namespaced_pod(target_namespace)

            for pod in pods.items:
                pod_name = pod.metadata.name
                if pod_name.startswith(key):
                    try:
                        v1.delete_namespaced_pod(name=pod_name, namespace=target_namespace)
                        print(f"Pod {pod_name} deleted successfully.")
                    except Exception as e:
                        print(f"Error deleting pod {pod_name}: {e}")


def main():
    if os.path.isfile(kubeconfig_path):
        config.load_kube_config()
    else:
        config.load_incluster_config()

    try:
        for namespace in ["control-plane-nexus", "user-management", "control-plane"]:
            if os.environ['CP_BACKUP_NAME'] and namespace == "control-plane":
                backup_name = os.environ['CP_BACKUP_NAME']
            elif os.environ['CP_NEXUS_BACKUP_NAME'] and namespace == "control-plane-nexus":
                backup_name = os.environ['CP_NEXUS_BACKUP_NAME']
            elif os.environ['UM_BACKUP_NAME'] and namespace == "user-management":
                backup_name = os.environ['UM_BACKUP_NAME']
            else:
                backup_name = fetch_latest_backup_name(client.CustomObjectsApi(), backup, namespace)
            print(f"Backup - {backup_name} for namespace {namespace}")
            if namespace == "control-plane":
                restore_control_plane_namespace(client, namespace, backup_name)
            else:
                delete_namespace(client, namespace)
                restore_namespace(client, namespace, backup_name)
        replace_secrets(client)
    except ApiException as e:
        print(f"Error updating values: {e}")


if __name__ == "__main__":
    main()
{{- end }}
